@@ -20,9 +20,15 @@
 //! # Ok::<(), io::Error>(())
 //! ```
 
+#[cfg(feature = "async")]
+mod r#async;
+
 pub mod header;
 mod reader;
 pub mod record;
 mod writer;
 
 pub use self::{header::Header, reader::Reader, record::Record, writer::Writer};
+
+#[cfg(feature = "async")]
+pub use self::r#async::{Reader as AsyncReader, Writer as AsyncWriter};
